# README

## Top-level Schemas

-   [Abstract](./abstract.md "This is an abstract schema") – `https://example.com/schemas/abstract`
-   [Arrays](./arrays.md "This is an example schema with examples for multiple array types and their constraints") – `https://example.com/schemas/arrays`
-   [Complex References](./complex.md "This is an example schema that uses types defined in other schemas") – `https://example.com/schemas/complex`
-   [Constant Types](./constants.md "This is an example schema with examples for properties with constant values") – `https://example.com/schemas/constants`
-   [Custom](./custom.md "This is an extensible schema") – `https://example.com/schemas/custom`
-   [Deeply Extending](./deepextending.md "This is an extending schema") – `https://example.com/schemas/deepextending`
-   [Definitions](./definitions.md "This is an example of using a definitions object within a schema") – `https://example.com/schemas/definitions`
-   [Enumerated ](./enums.md "This is an example schema with examples for properties with enum values") – `https://example.com/schemas/enums`
-   [Example](./example.md "This is an example schema with examples") – `https://example.com/schemas/example`
-   [Examples](./examples.md "This is an example schema with multiple examples") – `https://example.com/schemas/examples`
-   [Extending](./extending.md "This is an extending schema") – `https://example.com/schemas/extending`
-   [Extensible](./extensible.md "This is an extensible schema") – `https://example.com/schemas/extensible`
-   [Identifiable](./identifiable.md "This is a very simple example of a JSON schema") – `https://example.com/schemas/identifiable`
-   [Join Types](./join.md "This is an example of a JSON schema with only a join type key") – `https://example.com/schemas/join`
-   [Nested Object](./nestedobj.md) – `https://example.com/schemas/nestedobject`
-   [Pattern Properties](./pattern.md "This is an example of a JSON schema with only a patternProperties key") – `https://example.com/schemas/pattern`
-   [Simple](./simple.md "This is a very simple example of a JSON schema") – `https://example.com/schemas/simple`
-   [Simple Types](./simpletypes.md "This is an example schema with examples for multiple types and their constraints") – `https://example.com/schemas/simpletypes`
-   [Stabilizing](./stabilizing.md "This is a schema which is currently in the stabilizing status") – `https://example.com/schemas/stabilizing`
-   [Subdir](./subdir.md "A schema in a sub directory") – `https://example.com/schemas/subdir/subdir`
-   [Type Arrays](./typearrays.md "This schema test type arrays and nullable types") – `https://example.com/schemas/typearrays`

## Other Schemas

### Objects

-   [Enumerated (Nested)](./enums-properties-enumerated-nested.md "This is an example schema with examples for properties of nested objects with enum values") – `https://example.com/schemas/enums#/properties/nested`
-   [Simple](./complex-properties-simple.md "This is a very simple example of a JSON schema") – `https://example.com/schemas/simple#/properties/refnamed`
-   [Simple](./complex-properties-simple-1.md "This is a very simple example of a JSON schema") – `https://example.com/schemas/simple#/properties/refrefed`
-   [Simple](./complex-properties-refobj-properties-simple.md "This is a very simple example of a JSON schema") – `https://example.com/schemas/simple#/properties/refobj/properties/foo`
-   [Simple](./complex-properties-refnestedobj-properties-refobj-properties-simple.md "This is a very simple example of a JSON schema") – `https://example.com/schemas/simple#/properties/refnestedobj/properties/refobj/properties/foo`
-   [Untitled object in Abstract](./abstract-definitions-first.md) – `https://example.com/schemas/abstract#/definitions/first`
-   [Untitled object in Abstract](./abstract-definitions-second.md) – `https://example.com/schemas/abstract#/definitions/second`
-   [Untitled object in Arrays](./arrays-properties-objectlist-items.md) – `https://example.com/schemas/arrays#/properties/objectlist/items`
-   [Untitled object in Arrays](./arrays-properties-jointypelist-items-oneof-0.md "A simple string") – `https://example.com/schemas/arrays#/properties/JoinTypelist/items/oneOf/0`
-   [Untitled object in Arrays](./arrays-properties-jointypelist-items-oneof-1.md "Another simple string") – `https://example.com/schemas/arrays#/properties/JoinTypelist/items/oneOf/1`
-   [Untitled object in Complex References](./complex-properties-refabstract.md) – `https://example.com/schemas/complex#/properties/refabstract`
-   [Untitled object in Complex References](./complex-properties-refnestedobj-properties-refobj.md) – `https://example.com/schemas/complex#/properties/refnestedobj/properties/refobj`
-   [Untitled object in Complex References](./complex-properties-refobj.md) – `https://example.com/schemas/complex#/properties/refobj`
-   [Untitled object in Complex References](./complex-properties-refnestedobj.md) – `https://example.com/schemas/complex#/properties/refnestedobj`
-   [Untitled object in Join Types](./join-oneof-0-not-oneof-0.md "A simple string") – `https://example.com/schemas/join#/oneOf/0/not/oneOf/0`
-   [Untitled object in Join Types](./join-oneof-0-not-oneof-1.md "Another simple string") – `https://example.com/schemas/join#/oneOf/0/not/oneOf/1`
-   [Untitled object in Join Types](./join-oneof-1-allof-0.md "A simple string") – `https://example.com/schemas/join#/oneOf/1/allOf/0`
-   [Untitled object in Join Types](./join-oneof-2-anyof-0.md "Another simple string") – `https://example.com/schemas/join#/oneOf/2/anyOf/0`
-   [Untitled object in Nested Object](./nestedobj-properties-settings.md "settings") – `https://example.com/schemas/nestedobject#/properties/settings`
-   [Untitled object in Nested Object](./nestedobj-properties-settings-properties-collaborators.md "collaborators") – `https://example.com/schemas/nestedobject#/properties/settings/properties/collaborators`

### Arrays

-   [Untitled array in Arrays](./arrays-properties-intlist.md "This is an array") – `https://example.com/schemas/arrays#/properties/intlist`
-   [Untitled array in Arrays](./arrays-properties-tuple.md "This is an array of two values, one positive, one negative") – `https://example.com/schemas/arrays#/properties/tuple`
-   [Untitled array in Arrays](./arrays-properties-listlist.md "This is an array of arrays") – `https://example.com/schemas/arrays#/properties/listlist`
-   [Untitled array in Arrays](./arrays-properties-listlist-items.md) – `https://example.com/schemas/arrays#/properties/listlist/items`
-   [Untitled array in Arrays](./arrays-properties-stringlistlist.md "This is an array of arrays of strings") – `https://example.com/schemas/arrays#/properties/stringlistlist`
-   [Untitled array in Arrays](./arrays-properties-stringlistlist-items.md) – `https://example.com/schemas/arrays#/properties/stringlistlist/items`
-   [Untitled array in Arrays](./arrays-properties-list.md "This is an array") – `https://example.com/schemas/arrays#/properties/list`
-   [Untitled array in Arrays](./arrays-properties-boollist.md "This is an array") – `https://example.com/schemas/arrays#/properties/boollist`
-   [Untitled array in Arrays](./arrays-properties-numlist.md "This is an array") – `https://example.com/schemas/arrays#/properties/numlist`
-   [Untitled array in Arrays](./arrays-properties-coordinatelist.md "This is an array of coordinates in three-dimensional space") – `https://example.com/schemas/arrays#/properties/coordinatelist`
-   [Untitled array in Arrays](./arrays-properties-coordinatelist-items.md "A coordinate, specified by x, y, and z values") – `https://example.com/schemas/arrays#/properties/coordinatelist/items`
-   [Untitled array in Arrays](./arrays-properties-objectlist.md "An array of simple objects") – `https://example.com/schemas/arrays#/properties/objectlist`
-   [Untitled array in Arrays](./arrays-properties-jointypelist.md "An array of simple objects") – `https://example.com/schemas/arrays#/properties/JoinTypelist`
